@@ -13,6 +13,8 @@
 char *uri_decoded_copy (const char *str, int length, int *decoded_length);
 char *soup_uri_to_string_internal (SoupURI *uri, gboolean just_path_and_query,
 				   gboolean force_port);
+gboolean soup_uri_is_http (SoupURI *uri, char **aliases);
+gboolean soup_uri_is_https (SoupURI *uri, char **aliases);
 
 guint soup_socket_handshake_sync  (SoupSocket         *sock,
 				   GCancellable       *cancellable);
